@@ -4,43 +4,35 @@
 
     Highlight code blocks using Pygments.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2022 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import html
-import warnings
 from functools import partial
 from importlib import import_module
+from typing import Any, Dict
 
+from packaging import version
+from pygments import __version__ as pygmentsversion
 from pygments import highlight
 from pygments.filters import ErrorToken
+from pygments.formatter import Formatter
 from pygments.formatters import HtmlFormatter, LatexFormatter
 from pygments.lexer import Lexer
-from pygments.lexers import get_lexer_by_name, guess_lexer
-from pygments.lexers import PythonLexer, Python3Lexer, PythonConsoleLexer, \
-    CLexer, TextLexer, RstLexer
+from pygments.lexers import (CLexer, Python3Lexer, PythonConsoleLexer, PythonLexer, RstLexer,
+                             TextLexer, get_lexer_by_name, guess_lexer)
+from pygments.style import Style
 from pygments.styles import get_style_by_name
 from pygments.util import ClassNotFound
 
-from sphinx.deprecation import RemovedInSphinx30Warning
-from sphinx.ext import doctest
 from sphinx.locale import __
-from sphinx.pygments_styles import SphinxStyle, NoneStyle
-from sphinx.util import logging
-from sphinx.util.texescape import tex_hl_escape_map_new
-
-if False:
-    # For type annotation
-    from typing import Any, Dict  # NOQA
-    from pygments.formatter import Formatter  # NOQA
-    from pygments.style import Style  # NOQA
-
+from sphinx.pygments_styles import NoneStyle, SphinxStyle
+from sphinx.util import logging, texescape
 
 logger = logging.getLogger(__name__)
 
-lexers = {}  # type: Dict[str, Lexer]
-lexer_classes = {
+lexers: Dict[str, Lexer] = {}
+lexer_classes: Dict[str, Lexer] = {
     'none': partial(TextLexer, stripnl=False),
     'python': partial(PythonLexer, stripnl=False),
     'python3': partial(Python3Lexer, stripnl=False),
@@ -48,7 +40,7 @@ lexer_classes = {
     'pycon3': partial(PythonConsoleLexer, python3=True, stripnl=False),
     'rest': partial(RstLexer, stripnl=False),
     'c': partial(CLexer, stripnl=False),
-}  # type: Dict[str, Lexer]
+}
 
 
 escape_hl_chars = {ord('\\'): '\\PYGZbs{}',
@@ -60,6 +52,20 @@ escape_hl_chars = {ord('\\'): '\\PYGZbs{}',
 _LATEX_ADD_STYLES = r'''
 \renewcommand\PYGZsq{\textquotesingle}
 '''
+# fix extra space between lines when Pygments highlighting uses \fcolorbox
+# add a {..} to limit \fboxsep scope, and force \fcolorbox use correct value
+# cf pygments #1708 which makes this unneeded for Pygments > 2.7.4
+_LATEX_ADD_STYLES_FIXPYG = r'''
+\makeatletter
+% fix for Pygments <= 2.7.4
+\let\spx@original@fcolorbox\fcolorbox
+\def\spx@fixpyg@fcolorbox{\fboxsep-\fboxrule\spx@original@fcolorbox}
+\def\PYG#1#2{\PYG@reset\PYG@toks#1+\relax+%
+             {\let\fcolorbox\spx@fixpyg@fcolorbox\PYG@do{#2}}}
+\makeatother
+'''
+if version.parse(pygmentsversion).release <= (2, 7, 4):
+    _LATEX_ADD_STYLES += _LATEX_ADD_STYLES_FIXPYG
 
 
 class PygmentsBridge:
@@ -68,25 +74,20 @@ class PygmentsBridge:
     html_formatter = HtmlFormatter
     latex_formatter = LatexFormatter
 
-    def __init__(self, dest='html', stylename='sphinx', trim_doctest_flags=None):
-        # type: (str, str, bool) -> None
+    def __init__(self, dest: str = 'html', stylename: str = 'sphinx',
+                 latex_engine: str = None) -> None:
         self.dest = dest
+        self.latex_engine = latex_engine
 
         style = self.get_style(stylename)
-        self.formatter_args = {'style': style}  # type: Dict[str, Any]
+        self.formatter_args: Dict[str, Any] = {'style': style}
         if dest == 'html':
             self.formatter = self.html_formatter
         else:
             self.formatter = self.latex_formatter
             self.formatter_args['commandprefix'] = 'PYG'
 
-        self.trim_doctest_flags = trim_doctest_flags
-        if trim_doctest_flags is not None:
-            warnings.warn('trim_doctest_flags option for PygmentsBridge is now deprecated.',
-                          RemovedInSphinx30Warning, stacklevel=2)
-
-    def get_style(self, stylename):
-        # type: (str) -> Style
+    def get_style(self, stylename: str) -> Style:
         if stylename is None or stylename == 'sphinx':
             return SphinxStyle
         elif stylename == 'none':
@@ -97,27 +98,12 @@ class PygmentsBridge:
         else:
             return get_style_by_name(stylename)
 
-    def get_formatter(self, **kwargs):
-        # type: (Any) -> Formatter
+    def get_formatter(self, **kwargs: Any) -> Formatter:
         kwargs.update(self.formatter_args)
         return self.formatter(**kwargs)
 
-    def unhighlighted(self, source):
-        # type: (str) -> str
-        warnings.warn('PygmentsBridge.unhighlighted() is now deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-        if self.dest == 'html':
-            return '<pre>' + html.escape(source) + '</pre>\n'
-        else:
-            # first, escape highlighting characters like Pygments does
-            source = source.translate(escape_hl_chars)
-            # then, escape all characters nonrepresentable in LaTeX
-            source = source.translate(tex_hl_escape_map_new)
-            return '\\begin{Verbatim}[commandchars=\\\\\\{\\}]\n' + \
-                   source + '\\end{Verbatim}\n'
-
-    def get_lexer(self, source, lang, opts=None, force=False, location=None):
-        # type: (str, str, Dict, bool, Any) -> Lexer
+    def get_lexer(self, source: str, lang: str, opts: Dict = None,
+                  force: bool = False, location: Any = None) -> Lexer:
         if not opts:
             opts = {}
 
@@ -133,11 +119,6 @@ class PygmentsBridge:
                 lang = 'pycon3'
             else:
                 lang = 'python3'
-        elif lang == 'guess':
-            try:
-                lexer = guess_lexer(source)
-            except Exception:
-                lexer = lexers['none']
 
         if lang in lexers:
             # just return custom lexers here (without installing raiseonerror filter)
@@ -147,7 +128,7 @@ class PygmentsBridge:
         else:
             try:
                 if lang == 'guess':
-                    lexer = guess_lexer(lang, **opts)
+                    lexer = guess_lexer(source, **opts)
                 else:
                     lexer = get_lexer_by_name(lang, **opts)
             except ClassNotFound:
@@ -160,18 +141,13 @@ class PygmentsBridge:
 
         return lexer
 
-    def highlight_block(self, source, lang, opts=None, force=False, location=None, **kwargs):
-        # type: (str, str, Dict, bool, Any, Any) -> str
+    def highlight_block(self, source: str, lang: str, opts: Dict = None,
+                        force: bool = False, location: Any = None, **kwargs: Any) -> str:
         if not isinstance(source, str):
             source = source.decode()
 
         lexer = self.get_lexer(source, lang, opts, force, location)
 
-        # trim doctest options if wanted
-        if isinstance(lexer, PythonConsoleLexer) and self.trim_doctest_flags:
-            source = doctest.blankline_re.sub('', source)
-            source = doctest.doctestopt_re.sub('', source)
-
         # highlight via Pygments
         formatter = self.get_formatter(**kwargs)
         try:
@@ -192,10 +168,10 @@ class PygmentsBridge:
         if self.dest == 'html':
             return hlsource
         else:
-            return hlsource.translate(tex_hl_escape_map_new)
+            # MEMO: this is done to escape Unicode chars with non-Unicode engines
+            return texescape.hlescape(hlsource, self.latex_engine)
 
-    def get_stylesheet(self):
-        # type: () -> str
+    def get_stylesheet(self) -> str:
         formatter = self.get_formatter()
         if self.dest == 'html':
             return formatter.get_style_defs('.highlight')
