@@ -2,7 +2,7 @@
     sphinx.testing.restructuredtext
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2022 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -21,11 +21,13 @@ def parse(app: Sphinx, text: str, docname: str = 'index') -> nodes.document:
     """Parse a string as reStructuredText with Sphinx application."""
     try:
         app.env.temp_data['docname'] = docname
+        reader = SphinxStandaloneReader()
+        reader.setup(app)
         parser = RSTParser()
         parser.set_application(app)
         with sphinx_domains(app.env):
             return publish_doctree(text, path.join(app.srcdir, docname + '.rst'),
-                                   reader=SphinxStandaloneReader(app),
+                                   reader=reader,
                                    parser=parser,
                                    settings_overrides={'env': app.env,
                                                        'gettext_compact': True})
